﻿-- Author      : RisM
-- Create Date : 6/28/2009 3:54:18 PM

local SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)

SpeakinSpell:PrintLoading("utils.lua")

-------------------------------------------------------------------------------
-- UTILITY FUNCTIONS
-------------------------------------------------------------------------------


--TODO: consider using this function to make code more readable, but slower
function SpeakinSpell:RemoveEmptyTables( t )
	for key,value in pairs( t ) do
		if type(value) == "table" then
			self:RemoveEmptyTables( value )
			if self:IsTableEmpty( value ) then
				t[key] = nil
			end
		end
	end
end


-- Import new contents from Source into existing table Dest
function SpeakinSpell:AddTable(Dest, Source)
	-- create the destination if it doesn't exist
	if Dest == nil then
		Dest = {}
	end
	assert( type(Dest) == "table" )
	
	-- if there's no source, then there's nothing to add
	if Source == nil then
		-- nothing to add
		return Dest
	end
	
	-- add the contents of Source, one item at a time
	for k,v in pairs(Source) do
		-- dig into embedded tables to add those too, not just create references
		if type(v) == "table" then
			-- NOTE: safety checks above will ensure we create Dest[k] if necessary
			Dest[k] = self:AddTable(Dest[k], v)
		else
			Dest[k] = v
		end
	end
	
	return Dest
end


-- create a deep copy of the table and return it
function SpeakinSpell:CopyTable(SourceTable)
	-- adding SourceTable to a new empty table will create a copy
	return self:AddTable(nil, SourceTable)
end

-- add an array of strings
function SpeakinSpell:AddStringArray(Dest, Source)
	for _,s in ipairs(Source) do
		tinsert( Dest, s )
	end
end



-- make sure there are no empty strings or empty array indices in the string list
-- also remove duplicates
function SpeakinSpell:StringArray_Compress( List )
	-- the easiest thing to do is make a new list
	-- build a reverse lookup table of the target array first, to search for duplicates
	-- TODOFUTURE: could do this a faster way i'm sure
	local NewList = {}
	local UniqueCheck = {}
	for i,s in pairs( List ) do
		if (s ~= "") and (not UniqueCheck[s]) then
			local ret = tinsert( NewList, s )
			UniqueCheck[ s ] = true
		end
	end
	return NewList
end


function SpeakinSpell:IsTableEmpty( t )
	if not t then
		return true
	end
	for _,_ in pairs(t) do
		return false
	end
	return true
end



function SpeakinSpell:GetGuildName()
	local guildName = GetGuildInfo( UnitName("player") )
	if guildName then
		return guildName
	else
		return L["Your friends"]
	end
end


function SpeakinSpell:GetPlayerFullTitleName()
	local title = GetTitleName( GetCurrentTitle() )
	-- if we don't have a title, then self:StartsWith will error on the nil value
	if not title then
		return UnitName("player")
	end
	-- we have a title, so decide if it comes first or last
	-- blizzard does this by setting a space at the beginning or end for easier string concatenation
	if self:StartsWith(title, " ") then
		return UnitName("player")..tostring(title)
	else
		return title..UnitName("player")
	end
end


function SpeakinSpell:GetDefaultTarget(ShowDebugMsg)
	-- try the currently selected target
	local target = UnitName("target")
	if target then
		if ShowDebugMsg then self:DebugMsg(nil,"<target> is unknown, using <selected>:"..tostring(target)) end
		return target
	end
	
	-- nobody is selected, try focus
	target = UnitName("focus")
	if target then
		if ShowDebugMsg then self:DebugMsg(nil,"<target> is unknown, using <focus>:"..tostring(target)) end
		return target
	end
	
	-- nobody on focus, try mouseover
	target = UnitName("mouseover")
	if target then
		if ShowDebugMsg then self:DebugMsg(nil,"<target> is unknown, using <mouseover>:"..tostring(target)) end
		return target
	end
	
	-- assume self-cast ability
	target = UnitName("player")
	if ShowDebugMsg then self:DebugMsg(nil,"<target> is unknown, using <player>:"..tostring(target)) end
	
	return target
end


function SpeakinSpell:TitleCase(s)
	local first = string.sub(s,1,1)
	local rest = string.sub(s,2,-1)

	first = string.upper(first)
	
	rest = string.gsub(rest, " %a",
		function(match) 
			return string.upper(match)
		end
	)
	
	local result = tostring(first)..tostring(rest)
	return result
end


function SpeakinSpell:StartsWith(s, prefix)
	start = string.sub(s, 1, string.len(prefix))
	return (start == prefix)
end


function SpeakinSpell:EndsWith(s, suffix)
	strend = string.sub(s, -1 * string.len(suffix))
	return (strend == suffix)
end



function SpeakinSpell:IsInWintergraspBattle()
	--only applies to active battles
	
	--we only care if a battle is in progress
	--if we're in WG for VOA PVE and the battle is over, switch to raid settings
	if (GetWintergraspWaitTime() ~= nil) then
		-- a battle is not in progress
		return false
	end
	
	--TODOFUTURE: this has a localization problem since it does a string comparison
	-- unfortunately there doesn't appear to be a locale-independant way to test for wintergrasp
	if GetZoneText() == L["Wintergrasp GetZoneText"] then
		return true
	else
		return false
	end
end


function SpeakinSpell:GetRandomTableEntry( t, last )
	if not t then return nil end
	
	local max = #(t)
	if not (max >= 1) then return nil end
	
	local n = math.random(1, max);
	local sel = t[n]
	
	-- avoid repeating the same message we used last time
	-- unless there's only one in the list
	-- or we don't have a 'last'
	if last and (max > 1) then
		while sel == last do
			n = math.random(1, max);
			sel = t[n]
		end
	end
	
	return sel
end


function SpeakinSpell:IsInBG()
	-- simple check
	if (GetNumBattlefieldScores() > 0) then
		return true
	end
	
	--NOTE: GetNumBattlefieldScores returns 0 during the prep time
	-- in order to recognize that we're in the BG prep time and still treat it as a BG
	-- we have to check all possible queues to see if our status is now "active" in one of them
	local status, mapName, instanceID, minlevel, maxlevel;
	for i=1, MAX_BATTLEFIELD_QUEUES do
		status, mapName, instanceID, minlevel, maxlevel, teamSize = GetBattlefieldStatus(i);
		if status == "active" then
			return true
		end
	end
	
	-- doesn't look like we're in a BG
	return false
end


function SpeakinSpell:GetScenarioKey()
	local funcname = "GetScenarioKey"
	
	-- NOTE: the EventTableEntry.Channels table stores channel names that are compatible with SendChatMessage
	--	see also CurrentMessagesGUI_OnChannelSelect
	if (IsActiveBattlefieldArena()) then
		return "Arena"
	elseif self:IsInBG() then
		return "BG"
	elseif (self:IsInWintergraspBattle()) then --only applies to active battles
		-- NOTE: raid warnings not enabled for WG battles
		return "WG"
	elseif (GetNumRaidMembers() > 0) then
		-- override Raid with Leader if applicable
		if IsRaidLeader() then
			return "RaidLeader"
		elseif IsRaidOfficer() then
			return "RaidOfficer"
		else
			return "Raid"
		end
	elseif (GetNumPartyMembers() > 0) then
		-- override Party with Leader if applicable
		if IsPartyLeader() then
			return "PartyLeader"
		else
			return "Party"
		end
	else -- solo
		return "Solo"
	end
end



function SpeakinSpell:GetLanguageList()
	local funcname = "GetLanguageList"
	--self:DebugMsg(funcname, "entry")
	
	--Create and return a list of RP languages that are known to the player
	--NOTE: this does not include SpeakinSpell proprietary language filters like "Pirate" (TODOFUTURE: or does it?)
	--NOTE: format this table for use in the GUI
	local Langs = {}
	for i=1, GetNumLanguages() do
		local lang = GetLanguageByIndex(i)
		Langs[lang] = lang
	end
	return Langs
end


function SpeakinSpell:GetRacialLanguage()
	-- return GetLanguageByIndex( 2 ) -- appears to be non-deterministically ordered
	local Common = GetDefaultLanguage("player")
	for i=1, GetNumLanguages() do
		local lang = GetLanguageByIndex(i)
		if lang ~= Common then
			return lang
		end
	end
	return Common
end


-- if any key is missing from obj, it will be imported from DefaultObject
function SpeakinSpell:ValidateObject( obj, DefaultObject )
	for key,val in pairs(DefaultObject) do
		if obj[key] == nil then -- NOTE: don't replace "false" values
			if type( DefaultObject[key] ) == "table" then
				obj[key] = self:CopyTable(DefaultObject[key])
			else
				obj[key] = DefaultObject[key]
			end
		--no, this would bring in whole lists of things like messages
		--elseif type( DefaultObject[key] ) == "table" then
		--	self:ValidateObject( obj[key], DefaultObject[key] )
		end
	end
end


function SpeakinSpell:GetAddonMemoryUsedString()
	UpdateAddOnMemoryUsage("SpeakinSpell")
	return string.format( L["Memory Used: %d kb"], GetAddOnMemoryUsage("SpeakinSpell") )
end


-- Load chat channel colors from the chat frame settings
-- NOTE: it takes a while for ChatTypeInfo to be populated, so this doesn't work too early during loading
--		OnVariablesLoaded, the data is still not available, so we have to call this late as-needed in the GUI
--		which is just as well to force us to get the latest settings if they've changed
function SpeakinSpell:LoadChatColorCodes()
	-- NOTE: new colors can be declared with empty strings
	--		keys must be valid chat type names for ChatTypeInfo[]
	SpeakinSpell.Colors.Channels = {
		BATTLEGROUND= "|cffff6b00",
		RAID		= "|cffff6b00",
		GUILD		= "|c4444ff44",
		PARTY		= "|cffaaeeff",
		EMOTE		= "|cffff7b34",
		YELL		= "|cffff2f32",
		RAID_WARNING= "|cffffdbad",
		RAID_BOSS_WHISPER= "",
	}
	
	-- load the game's built-in channel colors from ChatTypeInfo
	for channel,_ in pairs( SpeakinSpell.Colors.Channels ) do
		local info = ChatTypeInfo[channel]
		if info then
			if nil == info.a then
				info.a = 1
			end
			SpeakinSpell.Colors.Channels[channel] = string.format( "|c%02x%02x%02x%02x", 255*info.a, 255*info.r, 255*info.g, 255*info.b )
		end
	end
	
	-- create colors code strings for the SpeakinSpell-created special channels
	if	SpeakinSpellSavedData and 
		SpeakinSpellSavedData.Colors and
		SpeakinSpellSavedData.Colors.Channels then
		for channel,info in pairs( SpeakinSpellSavedData.Colors.Channels ) do
			if nil == info.a then
				info.a = 1
			end
			SpeakinSpell.Colors.Channels[channel] = string.format( "|c%02x%02x%02x%02x", 255*info.a, 255*info.r, 255*info.g, 255*info.b )
		end
	end
end



function SpeakinSpell:StringColorCodeToTable(cc)
	-- cc is a string color code "|caarrggbb" example: "|cff123456"
	-- returns the color code as a table: t.a, t.r, t.g, t.b 
	-- using numbers in the format used by the color picker GUI controls and blizzard APIs
	-- which is a floating point number 0-1 representing the fraction of intensity 0-255
	
	-- extract component colors from the string escape sequence format "|caarrggbb"
	-- the "0x" is needed to make LUA interpret the string to number conversion correctly
	local aa = "0x"..string.sub( cc, 3, 4 )
	local rr = "0x"..string.sub( cc, 5, 6 )
	local gg = "0x"..string.sub( cc, 7, 8 )
	local bb = "0x"..string.sub( cc, 9, 11)
	
	-- convert string hex int "0xff" to floating point fraction or % of 0-255
	-- and store results in a table
	local t = {
		a = aa/255,
		r = rr/255,
		g = gg/255,
		b = bb/255,
	}
	
	return t
end



function SpeakinSpell:DamageSchoolCodeNumberToString(number)
	local Schools = {
		[1] = L["Physical"],
		[2] = L["Holy"],
		[4] = L["Fire"],
		[8] = L["Nature"],
		[12] = L["Firestorm"],
		[16] = L["Frost"],
		[20] = L["Frostfire"],
		[24] = L["Froststorm"],
		[32] = L["Shadow"],
		[40] = L["Shadowstorm"],
		[48] = L["Shadowfrost"],
		[64] = L["Arcane"],
		[68] = L["Spellfire"],
	}
	if Schools[number] then
		return Schools[number]
	else
		return L["Unknown Damage Type"]
	end
end

